package com.jnssd.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import springfox.documentation.builders.*;
import springfox.documentation.oas.annotations.EnableOpenApi;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.Docket;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <h3>spring-boot-openapi</h3>
 * <p>配置openapi</p>
 *
 * @author zxj
 * @since 2023-10-13 14:29:58
 */

@Configuration
@EnableOpenApi
public class SwaggerOpenApiConfig {

    @Bean
    public Docket api() {
        return new Docket(DocumentationType.OAS_30)
                .apiInfo(apiInfo())
                .select()
                // 扫描特定包
                // 扫描所有有注解的api，用这种方式更灵活
                // .apis(RequestHandlerSelectors.withMethodAnnotation(ApiOperation.class))
                //.apis(RequestHandlerSelectors.any())
                .apis(RequestHandlerSelectors.basePackage("com.jnssd"))
                .paths(PathSelectors.any())
                .build()
                .securitySchemes(initSecuritySchemeList())
                .securityContexts(Collections.singletonList(securityContext()));
    }

    public ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("Swagger项目测试")
                .description("novel项目接口文档")
                .build();
    }

    private List<SecurityScheme> initSecuritySchemeList() {

        List<SecurityScheme> list = new ArrayList<>();
        list.add(httpAuthenticationScheme());
        list.add(securitySchemeApiKey());
        list.add(securitySchemeOpenIdConnect());

        // 配置oauth2的几种模式
        list.add(securitySchemeOauth2ClientCredentials());
        list.add(securitySchemeOauth2implicit());
        list.add(securitySchemeOauth2Password());
        list.add(securitySchemeOauth2AuthorizationCode());
        return list;
    }

    private SecurityScheme httpAuthenticationScheme() {
        return HttpAuthenticationScheme.JWT_BEARER_BUILDER.name("JWT的值").build();
    }

    // ApiKey模式
    private SecurityScheme securitySchemeApiKey() {
        return new ApiKey("Authorization授权", "Authorization", "header");
    }

    // OpenIdConnect
    private SecurityScheme securitySchemeOpenIdConnect() {
       return new OpenIdConnectSchemeBuilder()
               .name("OpenId授权")
               .description("OpenIdConnect授权配置")
               .openIdConnectUrl("/oauth/authorize")
               .build();
    }

    // 客户端模式
    private SecurityScheme securitySchemeOauth2ClientCredentials() {
        return OAuth2Scheme.OAUTH2_CLIENT_CREDENTIALS_FLOW_BUILDER
                .name("客户端模式")
                .tokenUrl("/oauth/authorize")
                .scopes(scopes())
                .build();
    }

    // 隐式模式
    private SecurityScheme securitySchemeOauth2implicit() {
        return OAuth2Scheme.OAUTH2_IMPLICIT_FLOW_BUILDER
                .name("简化模式")
                .authorizationUrl("/oauth/authorize")
                .scopes(scopes())
                .build();
    }

    // 密码模式
    private SecurityScheme securitySchemeOauth2Password() {
        return OAuth2Scheme.OAUTH2_PASSWORD_FLOW_BUILDER
                .name("密码模式")
                .tokenUrl("/oauth/token")
                .scopes(scopes())
                .build();
    }

    // 授权码模式
    private SecurityScheme securitySchemeOauth2AuthorizationCode() {
        return OAuth2Scheme.OAUTH2_AUTHORIZATION_CODE_FLOW_BUILDER
                .name("授权码模式")
                .authorizationUrl("/oauth/authorize")
                .tokenUrl("/oauth/token")
                .scopes(scopes())
                .build();
    }

    private List<AuthorizationScope> scopes() {
        List<AuthorizationScope> list = new ArrayList<>();
        list.add(new AuthorizationScope("read_scope", "Grants read access"));
        list.add(new AuthorizationScope("write_scope", "Grants write access"));
        list.add(new AuthorizationScope("admin_scope", "Grants read write and delete access"));
        return list;
    }

    private SecurityContext securityContext() {
        return SecurityContext.builder()
                .securityReferences(defaultAuth())
                .operationSelector(operationContext -> {
                    System.out.println("operationContext" + operationContext);
                    return !operationContext.httpMethod().name().equals("GET");
                })
                .build();
    }

    private List<SecurityReference> defaultAuth() {
        AuthorizationScope authorizationScope = new AuthorizationScope("global", "accessEverything");
        AuthorizationScope[] authorizationScopes = new AuthorizationScope[1];
        authorizationScopes[0] = authorizationScope;
        return Collections.singletonList(new SecurityReference("密码模式", authorizationScopes));
    }
}
